#!/bin/bash
#/////////////////////////////////////////////////////////////////////////
#//
#// (c) University of Southampton IT Innovation Centre, 2018
#//
#// Copyright in this software belongs to University of Southampton
#// IT Innovation Centre of Gamma House, Enterprise Road,
#// Chilworth Science Park, Southampton, SO16 7NS, UK.
#//
#// This software may not be used, sold, licensed, transferred, copied
#// or reproduced in whole or in part in any manner or form or in or
#// on any media by any person other than in accordance with the terms
#// of the Licence Agreement supplied with the software, or otherwise
#// without the prior written consent of the copyright owners.
#//
#// This software is distributed WITHOUT ANY WARRANTY, without even the
#// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#// PURPOSE, except where stated in the Licence Agreement supplied with
#// the software.
#//
#//      Created By :            Michael Boniface, Nikolay Stanchev
#//      Created Date :          13/12/2018
#//      Created for Project :   FLAME
#//
#/////////////////////////////////////////////////////////////////////////
apt-get update -o Acquire::CompressionTypes::Order::=gz
apt-get install libssl-dev -y

# install virtualenvwrapper to manage python environments - and check
echo "----> Installing Python3, Pip3 and curl"
apt-get install -y python3 python3-pip curl jq
update-alternatives --install /usr/bin/python python /usr/bin/python3 10
pip3 install --upgrade pip

echo "----> Installing virtualenv and wrapper"
pip3 install virtualenv
pip3 install virtualenvwrapper

## CLMC-SERVICE
## ----------------------------------------------------------------------------------
echo "----> Configuring virtualenvwrapper"
export WORKON_HOME=$HOME/.virtualenvs
source /usr/local/bin/virtualenvwrapper.sh

# check the mkvirtualenv with a return value of 1 if version comes back correctly
mkvirtualenv --version
if [ $? -ne 1 ] ; then
        echo "Failed: installing virtualenvwrapper"
		exit 1
fi

# create CLMC virtual environment - and check
echo "----> Making CLMC Python environment"
mkvirtualenv CLMC
if [ $? -ne 0 ] ; then
        echo "Failed: creating CLMC python environment"
		exit 1
fi

# switch the CLMC environment - and check
echo "----> Switching to use CLMC python environment"
workon CLMC
if [[ $? -ne 0 ]] ; then
        echo "Failed: switching to CLMC python environment"
		exit 1
fi

# navigate to the clmc-service folder and check
echo "----> Moving to CLMC service"
cd ${REPO_ROOT}/src/service
if [[ $? -ne 0 ]] ; then
        echo "Failed: could not find clmc-service"
		exit 1
fi

echo "----> Replicating CLMC Python environment"
pip3 install -r requirements.txt
if [[ $? -ne 0 ]] ; then
        echo "Failed: could not replicate CLMC Python environment from requirements.txt"
		exit 1
fi

# running tests using pytest
echo "----> Running tests"
pytest -v clmcservice
if [[ $? -ne 0 ]] ; then
        echo "Failed: CLMC service unit tests have failed."
		exit 1
fi
echo "----> Execution of unit tests passed successfully"

# install the service
echo "----> Installing CLMC web service"
pip3 install  .
if [[ $? -ne 0 ]] ; then
        echo "Failed: installing clmc-webservice"
		exit 1
fi

# create directory for CLMC service logs
echo "----> Creating CLMC web service log directory"
mkdir -p /var/log/flame/clmc

# move the graph pipeline script
cp ${REPO_ROOT}/scripts/clmc-service/graph-pipeline.sh /usr/local/bin/graph-pipeline.sh
chmod u+x /usr/local/bin/graph-pipeline.sh

# move the graph network topology script
cp ${REPO_ROOT}/scripts/clmc-service/graph-network-topology.sh /usr/local/bin/graph-network-topology.sh
chmod u+x /usr/local/bin/graph-network-topology.sh

# Install clmc as systemctl service
# -----------------------------------------------------------------------
mkdir -p /opt/flame/clmc
start_script_file="/opt/flame/clmc/start.sh"
echo "#!/bin/bash" > $start_script_file
echo "export WORKON_HOME=${HOME}/.virtualenvs" >> $start_script_file
echo "export SFEMC_FQDN=${SFEMC_FQDN}" >> $start_script_file
echo "export SDN_CONTROLLER_IP=${SDN_CONTROLLER_IP}" >> $start_script_file
echo "source /usr/local/bin/virtualenvwrapper.sh" >> $start_script_file
echo "workon CLMC" >> $start_script_file
echo "pserve ${REPO_ROOT}/src/service/production.ini &" >> $start_script_file

chmod 755 $start_script_file

file="/lib/systemd/system/flameclmc.service"
echo "[Unit]" > $file
echo "Description=flameclmc" >> $file
echo "After=${NETWORK_DEPENDENCY}" >> $file
echo "Requires=${NETWORK_DEPENDENCY}" >> $file
echo "" >> $file
echo "[Service]" >> $file
echo "Type=forking" >> $file
echo "ExecStart=${start_script_file}" >> $file
echo "" >> $file
echo "[Install]" >> $file
echo "WantedBy=multi-user.target" >> $file

systemctl daemon-reload
systemctl enable flameclmc.service
systemctl start flameclmc.service

# wait for the clmc service to start
while ! nc -z localhost 9080
do
  echo "Waiting for clmc service port 9080 to be ready on localhost..."
  sleep 5
done
